/*-
 * ============LICENSE_START=======================================================
 * guard
 * ================================================================================
 * Copyright (C) 2017 AT&T Intellectual Property. All rights reserved.
 * ================================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ============LICENSE_END=========================================================
 */

package org.onap.policy.guard;

import java.util.UUID;

import org.drools.core.WorkingMemory;
import org.onap.policy.drools.system.PolicyEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.att.research.xacml.api.DataTypeException;
import com.att.research.xacml.std.annotations.RequestParser;


public class CallGuardTask implements Runnable {
	
	private static final Logger logger = LoggerFactory.getLogger(CallGuardTask.class);
	WorkingMemory workingMemory;
	String restfulPdpUrl;
	String clname;
	String actor;
	String recipe;
	String target;
	String requestId;
	
    public CallGuardTask(String guardUrl, WorkingMemory wm, String cl, String act, String rec, String tar, String reqId) { 
    
    	restfulPdpUrl = guardUrl;
    	workingMemory = wm;
    	clname = cl;
    	actor = act;
    	recipe = rec;
    	requestId = reqId;
    	target = tar;
    }
    public void run() {
    	long startTime = System.nanoTime();
    	com.att.research.xacml.api.Request request = null;
    	
    	PolicyGuardXacmlRequestAttributes xacmlReq = new PolicyGuardXacmlRequestAttributes(clname, actor,  recipe, target, requestId);
    	
    	try {
    		request = RequestParser.parseRequest(xacmlReq);
		} catch (IllegalArgumentException | IllegalAccessException | DataTypeException e) {
			logger.error("CallGuardTask.run threw: {}", e);
		} 
    	
		
  		logger.debug("\n********** XACML REQUEST START ********");
		logger.debug("{}", request);
		logger.debug("********** XACML REQUEST END ********\n");
		
		String guardUrl = PolicyEngine.manager.getEnvironmentProperty("guard.url");
		String guardDecision = null;
		
		//
		// Check if guard url property exists
		//
		if(guardUrl != null){
			guardDecision = PolicyGuardXacmlHelper.callPDP(guardUrl, xacmlReq);
		}

		logger.debug("\n********** XACML RESPONSE START ********");
		logger.debug("{}", guardDecision);
		logger.debug("********** XACML RESPONSE END ********\n");

		//
		// Check if the restful call was unsuccessful or property doesn't exist
		//
		if(guardDecision == null){
			logger.error("********** XACML FAILED TO CONNECT ********");
			guardDecision = "Indeterminate";
		}

		PolicyGuardResponse guardResponse = new PolicyGuardResponse(guardDecision, UUID.fromString(this.requestId), this.recipe);

		
		//
		//Create an artificial Guard response in case we didn't get a clear Permit or Deny
		//
		if(guardResponse.result.equals("Indeterminate")){
			guardResponse.operation = recipe;
			guardResponse.requestID = UUID.fromString(requestId);
		}
		
		long estimatedTime = System.nanoTime() - startTime;
		logger.debug("\n\n============ Guard inserted with decision {} !!! =========== time took: {} mili sec \n\n",
				guardResponse.result, (double)estimatedTime/1000/1000);
		workingMemory.insert(guardResponse);

    }

}
